/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2006, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wsf.stack.metro;

import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import javax.ejb.Stateless;
import javax.jws.WebMethod;
import javax.jws.WebService;
import javax.jws.soap.SOAPBinding.ParameterStyle;
import javax.jws.soap.SOAPBinding.Style;
import javax.jws.soap.SOAPBinding.Use;
import javax.xml.ws.WebFault;
import javax.xml.ws.WebServiceException;

import org.jboss.logging.Logger;
import org.jboss.wsf.spi.deployment.UnifiedVirtualFile;

import com.sun.xml.ws.api.server.Container;
import com.sun.xml.ws.api.server.InstanceResolver;
import com.sun.xml.ws.api.server.Invoker;
import com.sun.xml.ws.transport.http.ResourceLoader;
import com.sun.xml.ws.transport.http.DeploymentDescriptorParser.AdapterFactory;

/**
 * A copy of DeploymentDescriptorParser that externalizes
 * 
 * createInvoker(Class)
 * generateWrappers(Class, XMLStreamReader, ClassLoader)
 *
 * @author Thomas.Diesler@jboss.org
 * @author alessio.soldano@jboss.org
 * @since 10-May-2007
 */
public class DeploymentDescriptorParserJBWS<A> extends DeploymentDescriptorParserExt<A>
{
   private UnifiedVirtualFile archiveFile;
   
   public DeploymentDescriptorParserJBWS(ClassLoader cl, ResourceLoader loader, Container container, AdapterFactory<A> adapterFactory) throws MalformedURLException
   {
      super(cl, loader, container, adapterFactory);
   }

   @Override
   protected Invoker createInvoker(Class<?> implClass)
   {
      Invoker invoker;
      InstanceResolver<?> resolver = InstanceResolver.createDefault(implClass);
      if (implClass.isAnnotationPresent(Stateless.class))
      {
         invoker = new InvokerEJB3(resolver);
      }
      else
      {
         invoker = new InvokerJSE(resolver);
      }
      return invoker;
   }
   
   @Override
   protected void generateWrappers(Class<?> implementorClass, ClassLoader classLoader)
   {
      //Generate wrappers
      if (isWrapperGenerationEnabled() && isWrapperGenerationRequired(implementorClass, classLoader))
      {
         WsgenWrapperGenerator wrapperGenerator = new WsgenWrapperGenerator(System.out);
         ClassLoader newClassLoader = wrapperGenerator.generate(implementorClass.getName(), classLoader, archiveFile.toURL());
         Thread.currentThread().setContextClassLoader(newClassLoader);
      }
   }
   
   private boolean isWrapperGenerationEnabled()
   {
      boolean noWrappersGeneration = Boolean.getBoolean(Constants.NO_WRAPPERS_GENERATION);
      if (noWrappersGeneration)
         Logger.getLogger(this.getClass()).debug("Wrapper generation disabled");
      return !noWrappersGeneration;
   }
   
   private boolean isWrapperGenerationRequired(Class<?> endpoint, ClassLoader classLoader)
   {
      WebService webServiceAnnotation = endpoint.getAnnotation(WebService.class);
      if (webServiceAnnotation == null)
         return false;
      String wsdlLocation = webServiceAnnotation.wsdlLocation();
      if (wsdlLocation != null && !"".equals(wsdlLocation))
         return false; //provided wsdlLocation means a top-down (contract first) development is used, thus the user should use wsimport
      
      String endpointInterface = webServiceAnnotation.endpointInterface();
      javax.jws.soap.SOAPBinding soapBinding;
      if (endpointInterface == null || "".equalsIgnoreCase(endpointInterface))
      {
         if (isFaultWrapperGenerationRequired(endpoint))
            return true;
         soapBinding = endpoint.getAnnotation(javax.jws.soap.SOAPBinding.class);
      }
      else
      {
         //Using the SEI to get annotations
         Class<?> sei;
         try
         {
            sei = Class.forName(endpointInterface, true, classLoader);
         }
         catch (ClassNotFoundException e)
         {
            throw new WebServiceException(e);
         }
         if (isFaultWrapperGenerationRequired(sei))
            return true;
         soapBinding = sei.getAnnotation(javax.jws.soap.SOAPBinding.class);
      }
      if (soapBinding == null)
         return true; //no @SOAPBinding means default settings, ie. doc/lit wrapped
      return !(ParameterStyle.BARE.equals(soapBinding.parameterStyle()) || Style.RPC.equals(soapBinding.style()) || Use.ENCODED.equals(soapBinding.use()));
   }
   
   private boolean isFaultWrapperGenerationRequired(Class<?> clazz)
   {
      Method[] classMethods = clazz.getMethods();
      boolean webMethodAnnUsed = false;
      List<Method> webMethods = new LinkedList<Method>();
      for (Method m : classMethods)
      {
         if (m.getAnnotation(WebMethod.class) != null)
         {
            webMethods.add(m);
            webMethodAnnUsed = true;
         }
      }
      List<Method> methods = webMethodAnnUsed ? webMethods : Arrays.asList(classMethods);
      for (Method m : methods)
      {
         Class<?>[] exceptionClasses = m.getExceptionTypes();
         for (Class<?> exceptionClass : exceptionClasses)
         {
            if (exceptionClass.getAnnotation(WebFault.class) != null)
               return true;
         }
      }
      return false;
   }
   
   public void setArchiveFile(UnifiedVirtualFile file)
   {
      this.archiveFile = file;
   }
}
